/* *******************************************************************************************
The MIT License

Copyright (c) 2007 Steven G. Greenberg

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
******************************************************************************************* */

#pragma once

// This macro returns the "distance" between two indexes ("h" & "t" for head & tail)
// of a ring buffer of size "s". Generally equivalent to the "#of bytes in the ring buffer")
//
#define RINGDIST(h,t,s) (DWORD)((h >= t) ? (h-t) : (s - (t-h)))

struct RINGBUF
{
	DWORD	dwLog2Size;
	DWORD	dwBfrSize;
	DWORD	dwBfrMask;
	DWORD	dwBfrReloadGap;
	DWORD	dwBfrUsableSize;
	DWORD	dwHeadIdx;
	DWORD	dwTailIdx;
	BYTE*	pRingBuf;
};


class CFileCache2
{
public:
	CFileCache2(void);

	BOOL	Init(HANDLE hFile, DWORD dwLog2Size);

protected:

	QWORD	m_qwFilePos;
	QWORD	m_qwTotalFileSize;

	HANDLE	m_hFile;

	struct RINGBUF	m_bfr;

	DWORD BytesInCache(void)		{return(RINGDIST(m_bfr.dwHeadIdx, m_bfr.dwTailIdx, m_bfr.dwBfrSize)); }

	BOOL	BytesNeeded(DWORD dwCount);

	QWORD	GetDiskFilePos(void);

public:
	DWORD	ReadBytes(void* pBuf, DWORD dwCount);
	DWORD	SkipBytes(DWORD dwCount);
	QWORD	GetFileSize(void);

	QWORD	NextVarlenLe(DWORD dwBytes);
	DWORD	NextAsfVarlen(DWORD dwLenCode);
	QWORD	NextQWordLe(void);
	DWORD	NextDWordLe(void);
	WORD	NextWordLe(void);
	BYTE	NextByte(void);

	void	GetNextGuid(GUID* pGuid);

	QWORD	GetFilePos(void);

public:
	~CFileCache2(void);
};
