/* *******************************************************************************************
The MIT License

Copyright (c) 2007 Steven G. Greenberg

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
******************************************************************************************* */

#include <windows.h>

#include "constants.h"
#include "PutBits.h"

#define SETBITBYIDX(pb,n) pb[n/8] |= g_maskTbl[n%8];

const BYTE g_maskTbl[8] = {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};

CPutBits::CPutBits(BYTE* pBuf, DWORD dwBufLen)
{
	m_pBuf = pBuf;
	m_dwBufLen = dwBufLen;
	m_dwRunningIdx = 0;
	memset(pBuf, 0, dwBufLen);
}

CPutBits::~CPutBits(void)
{
}

void CPutBits::PutBits(DWORD dwNumBits, DWORD dwVal)
{
	PutBitsByIndex(m_dwRunningIdx, dwNumBits, dwVal);
	m_dwRunningIdx += dwNumBits;
}

void CPutBits::ResyncToNextByte(void)
{
	// this will skip the running bit index to the next "on-byte" boundary
	m_dwRunningIdx = 8 * ((m_dwRunningIdx + 7) / 8 );
}

void CPutBits::PutBitsByIndex(DWORD dwStartBitIdx, DWORD dwNumBits, DWORD dwVal)
{

	_ASSERT_(dwNumBits == 32 || dwVal < ((DWORD)0x01 << dwNumBits));

	if (dwNumBits <= 32)
	{
		dwVal = dwVal << (32 - dwNumBits);

		if ((dwStartBitIdx + dwNumBits) / 8 < m_dwBufLen)
		{
			for (DWORD dwIdx = dwStartBitIdx; dwIdx < dwStartBitIdx + dwNumBits; dwIdx++)
			{
				if (dwVal & 0x80000000)
					SETBITBYIDX(m_pBuf, dwIdx);
				
				dwVal *= 2;
			}
		}
		else
			_ASSERT_(0);
	}
	else
		_ASSERT_(0);
}


DWORD CPutBits::GetByteLen()
{
	return((m_dwRunningIdx+7) / 8);
}
