/* *******************************************************************************************
The MIT License

Copyright (c) 2007 Steven G. Greenberg

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
******************************************************************************************* */

// disable warning about deprecated string functions

#pragma warning( disable : 4996 )

#include <assert.h>

#ifdef DEBUG
#define _ASSERT_(x) assert(x)
#else
#define _ASSERT_(x) (void*)0
#endif

//#ifndef DEBUG
//#pragma warning( disable : 4100 )	// unref vars for above
//int _CrtDbgReport(int _ReportType, const char * _Filename, int _Linenumber, const char * _ModuleName, const char * _Format, ...);
//#endif

typedef unsigned __int64	QWORD;


//#define DBGW _CRT_WARN, __FILE__, __LINE__, NULL

#define IS_VALID_HANDLE(x)		((x) != NULL && (x) != INVALID_HANDLE_VALUE)

#define GUIDS_EQUAL(x,y) (memcmp(&(x), &(y), sizeof(x)) == 0)

#define DWLE_FOURCC(psz)	(*((DWORD*)psz))


#define MICROSECONDS_PER_SECOND		(1000 * 1000)	// for clarity
#define MAX_REASONABLE_HDR_SIZE		(100 * 1024)	// if the "header" bigger than 100K, the hell with it

#define MAX_WMV_STREAMS		128
#define MAX_EXPECTED_FRAMESIZE	(512 * 1024)	// really a max expected "object" size

// This could actually be calculated, but we'll just use a vary large value. They are usually a dozen or two bytes,
// though the whole "32 leaky bucket" thing *could* make for a pretty long one. Anyway, this is plenty.

#define	MAX_SEQ_PLUS_ENTRY_LEN		1024	// max poss #of bytes total in a sequence and entry point header

#define META_TAG_DATA_MAX			64		// arbitrarily limit each item to 64 characters plus null term

#define MAX_ERR_MSG_LEN				256		// currently only used by WLastErrStr() for system messages

#define SEQ_SC		0x0000010f	// VC1 Sequence Start Code
#define ENT_SC		0x0000010e	// VC1 Entry Point Start Code
#define FRM_SC		0x0000010d	// VC1 Frame Start Code
#define EOS_SC		0x0000010a	// VC1 End Of Sequence Code

#define MAX_LEAKY_BUCKET_VALS	32

#define LITTLE_ENDIAN

#ifdef LITTLE_ENDIAN
#define NTOHL(x)((((x) & 0xff000000) >> 24) | \
				 (((x) & 0x00ff0000) >> 8)  | \
				 (((x) & 0x0000ff00) << 8)  | \
				 (((x) & 0x000000ff) << 24))
#else
#define NTOHL(x) (x)
#endif

// Quick "verbosity" implementation by "level" only. This will probably get changed in the future.
#define LOG_TYPE_CONSOLE_ERROR		0
#define LOG_LEVEL_1					1
#define LOG_LEVEL_2					2
#define LOG_LEVEL_3					3

#define LOG_LEVEL_DEFAULT			2


enum CMND_LINE_FLAG
{
	CL_FLAG_HELP,
	CL_FLAG_NO_PAR,
	CL_FLAG_NO_FRAMERATE,
	CL_FLAG_NO_REPEAT_SEQUENCE_HEADER,
//	CL_FLAG_NO_END_OF_SEQUENCE_HEADER,
	CL_FLAG_VERBOSE_0,
	CL_FLAG_VERBOSE_1,
	CL_FLAG_VERBOSE_2,
	CL_FLAG_VERBOSE_3,
	CL_FLAG_END_OF_TABLE
};


// these are also used as integer indexes into an array, so change at your own risk
enum VC1_FRAMETYPE
{
	FRMTYPE_UNDEFINED = 0,
	FRMTYPE_FRM_I,
	FRMTYPE_FRM_P,
	FRMTYPE_FRM_B,
	FRMTYPE_FRM_BI,
	FRMTYPE_FRM_D,
	FRMTYPE_FLD_I_I,
	FRMTYPE_FLD_I_P,
	FRMTYPE_FLD_P_I,
	FRMTYPE_FLD_P_P,
	FRMTYPE_FLD_B_B,
	FRMTYPE_FLD_B_BI,
	FRMTYPE_FLD_BI_B,
	FRMTYPE_FLD_BI_BI,
	FRMTYPE_NUM_FRMTYPES
};

enum VC1_HDR_PARAMS
{
	__,			// used a where a conditional jump label would be when there is no conditional, i.e. means "NA"
	_____,		// as an entry to get compare value from, means "this entry" (if there even IS a conditional, else means "NA")
	_SEQUENCE_START_,
	PROFILE,
	LEVEL,
	COLORDIFF_FORMAT,
	FRMRTQ_POSTPROC,
	BITRTQ_POSTPROC,
	POSTPROCFLAG,
	MAX_CODED_WIDTH,
	MAX_CODED_HEIGHT,
	PULLDOWN,
	INTERLACE,
	TFCNTRFLAG,
	FINTERPFLAG,
    RESERVED,
	PSF,
	DISPLAY_EXT,
    DISP_HORIZ_SIZE,
    DISP_VERT_SIZE,
    ASPECT_RATIO_FLAG,
	ASPECT_RATIO,
	ASPECT_HORIZ_SIZE,
	ASPECT_VERT_SIZE,
	FRAMERATE_FLAG,
	FRAMERATEIND,
	FRAMERATENR,
    FRAMERATEDR,
	FRAMERATEEXP,
	COLOR_FORMAT_FLAG,
	COLOR_PRIM,
	TRANSFER_CHAR,
	MATRIX_COEF,
	HRD_PARAM_FLAG,
	HRD_NUM_LEAKY_BUCKETS,
	BIT_RATE_EXPONENT,
	BUFFER_SIZE_EXPONENT,
	BROKEN_LINK,
	_ENTRYPOINT_START_,
	CLOSED_ENTRY,
	PANSCAN_FLAG,
	REFDIST_FLAG,
	LOOPFILTER,
	FASTUVMC,
	EXTENDED_MV,
	DQUANT,
	VSTRANSFORM,
	OVERLAP,
    QUANTIZER,
    CODED_SIZE_FLAG,
	CODED_WIDTH,
	CODED_HEIGHT,
	EXTENDED_DMV,
	RANGE_MAPY_FLAG,
    RANGE_MAPY,
	RANGE_MAPUV_FLAG,
	RANGE_MAPUV,
	_LABEL1_,			// misc phony (zero bit) entry used conditional as a jump target
	_LEAKY_BUCKETS_,	// phony (zero bit) entry activates code outside "interpreter" (to fill up to 32 values)
	_HRD_FULLNESS_,		// phony (zero bit) entry activates code outside "interpreter" (to fill up to 32 values)
	_END_OF_SEQ_HDR_,	// phony (zero bit) entry marks end of Sequence Layer & start of Entry Point Layer parsing
	_DISPLAY_EXT_COND_,
	_END_,				// not really needed - precedes __EOT__ below & does nothing. Avoids an assert if jumping to end of table.
	__EOT__				// special phoney (zero bit) entry marks end of entire table
};

// A rather incomplete set of "conditionals" but all that turns out to be needed for the
// job at hand. It would be trivial to add more, "branch if less than", etc.
enum COMPARE_TYPE

{
	___,		// none
	BNE,		// branch if not equal
	BRA			// branch always
};

#define _ 0
